from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class PixClaim(Resource):
    """# PixClaim object
    PixClaims intend to transfer a PixKey from one account to another.
    When you initialize a PixClaim, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the created object.
    ## Parameters (required):
    - account_created [datetime.date, datetime.datetime or string]: opening Date or DateTime for the account claiming the PixKey. ex: "2022-01-01"
    - account_number [string]: number of the account claiming the PixKey. ex: "76543"
    - account_type [string]: type of the account claiming the PixKey. Options: "checking", "savings", "salary" or "payment"
    - branch_code [string]: branch code of the account claiming the PixKey. ex: "1234"
    - name [string]: holder's name of the account claiming the PixKey. ex: "Jamie Lannister"
    - tax_id [string]: holder's taxId of the account claiming the PixKey (CPF/CNPJ). ex: "012.345.678-90"
    - key_id [string]: id of the registered Pix Key to be claimed. Allowed keyTypes are CPF, CNPJ, phone number or email. ex: "+5511989898989"
    # Parameters (optional):
    - tags [list of strings, default []]: list of strings for tagging. ex: ["travel", "food"]
    ## Attributes (return-only):
    - id [string]: unique id returned when the PixClaim is created. ex: "5656565656565656"
    - bacen_id [string, default None]: unique transaction id returned from Central Bank. ex: "ccf9bd9c-e99d-999e-bab9-b999ca999f99"
    - status [string]: current PixClaim status. Options: "created", "failed", "delivered", "confirmed", "success", "canceled"
    - type [string]: type of Pix Claim. Options: "ownership", "portability"
    - key_type [string]: keyType of the claimed PixKey. Options: "CPF", "CNPJ", "phone" or "email"
    - flow [string]: direction of the Pix Claim. Options: "in" if you received the PixClaim or "out" if you created the PixClaim.
    - claimer_bank_code [string]: bank_code of the Pix participant that created the PixClaim. ex: "20018183"
    - claimed_bank_code [string]: bank_code of the account donating the PixKey. ex: "20018183"
    - created [datetime.datetime]: creation datetime for the PixClaim. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: update datetime for the PixClaim. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, account_created, account_number, account_type, branch_code, name, tax_id, key_id, tags=None, id=None,
                 bacen_id=None, status=None, type=None, key_type=None, flow=None, claimer_bank_code=None, claimed_bank_code=None,
                 created=None, updated=None):
        Resource.__init__(self, id=id)

        self.account_created = check_datetime(account_created)
        self.account_number = account_number
        self.account_type = account_type
        self.branch_code = branch_code
        self.name = name
        self.tax_id = tax_id
        self.key_id = key_id
        self.tags = tags
        self.bacen_id = bacen_id
        self.status = status
        self.type = type
        self.key_type = key_type
        self.flow = flow
        self.claimer_bank_code = claimer_bank_code
        self.claimed_bank_code = claimed_bank_code
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": PixClaim, "name": "PixClaim"}


def create(claim, user=None):
    """# Create a PixClaim object
    Create a PixClaim to request the transfer of a PixKey to an account
    hosted at other Pix participants in the Stark Infra API.
    ## Parameters (required):
    - claim [PixClaim object]: PixClaim object to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixClaim object with updated attributes.
    """
    return rest.post_single(resource=_resource, entity=claim, user=user)


def get(id, user=None):
    """# Retrieve a PixClaim object
    Retrieve a PixClaim object linked to your Workspace in the Stark Infra API by its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixClaim object that corresponds to the given id.
    """
    return rest.get_id(id=id, resource=_resource, user=user)


def query(limit=None, after=None, before=None, status=None, ids=None, bacen_id=None, type=None, key_type=None, key_id=None, flow=None, tags=None, user=None):
    """# Retrieve PixClaims
    Receive a generator of PixClaim objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "failed", "delivered", "confirmed", "success", "canceled"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - bacen_id [string, default None]: unique transaction id returned from Central Bank. ex: "ccf9bd9c-e99d-999e-bab9-b999ca999f99"
    - type [string, default None]: filter for the type of retrieved PixClaims. Options: "ownership" or "portability"
    - key_type [string, default None]: filter for the PixKey type of retrieved PixClaims. Options: "cpf", "cnpj", "phone", "email" and "evp"
    - key_id [string, default None]: filter PixClaims linked to a specific PixKey id. ex: "+5511989898989"
    - flow [string, default None]: direction of the Pix Claim. Options: "in" if you received the PixClaim or "out" if you created the PixClaim.
    - tags [list of strings, default None]: list of strings to filter retrieved objects. ex: ["travel", "food"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of PixClaim objects with updated attributes
    """

    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        bacen_id=bacen_id,
        type=type,
        key_type=key_type,
        key_id=key_id,
        flow=flow,
        tags=tags,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, ids=None, bacen_id=None, type=None, key_type=None, key_id=None, flow=None, tags=None, user=None):
    """# Retrieve paged PixClaims
    Receive a list of up to 100 PixClaim objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call.
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "failed", "delivered", "confirmed", "success", "canceled"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - bacen_id [string, default None]: unique transaction id returned from Central Bank. ex: "ccf9bd9c-e99d-999e-bab9-b999ca999f99"
    - type [string, default None]: filter for the type of retrieved PixClaims. Options: "ownership" or "portability"
    - key_type [string, default None]: filter for the PixKey type of retrieved PixClaims. Options: "cpf", "cnpj", "phone", "email" and "evp"
    - key_id [string, default None]: filter PixClaims linked to a specific PixKey id. Example: "+5511989898989"
    - flow [string, default None]: direction of the Pix Claim. Options: "in" if you received the PixClaim or "out" if you created the PixClaim.
    - tags [list of strings, default None]: list of strings to filter retrieved objects. ex: ["travel", "food"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of PixClaim objects with updated attributes and cursor to retrieve the next page of PixClaim objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        bacen_id=bacen_id,
        type=type,
        key_type=key_type,
        key_id=key_id,
        flow=flow,
        tags=tags,
        user=user,
    )


def update(id, status, reason=None, user=None):
    """# Update PixClaim entity
    Update a PixClaim parameters by passing id.
    ## Parameters (required):
    - id [string]: PixClaim id. ex: "5656565656565656"
    - status [string]: patched status for Pix Claim. Options: "confirmed" and "canceled"
    ## Parameters (optional):
    - reason [string, default: "userRequested"]: reason why the PixClaim is being patched. Options: "fraud", "userRequested", "accountClosure"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixClaim with updated attributes
    """
    payload = {
        "reason": reason,
        "status": status,
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)



from ..utils import rest
from starkinfra import request_methods_prefix


def get(path, query=None, user=None):
    """# Retrieve any StarkInfra resource
    Receive a json of resources previously created in StarkInfra's API
    ## Parameters (required):
    - path [string]: StarkInfra resource's route. ex: "/pix-request/"
    - query [dict, default None]: Query parameters. ex: {"limit": 1, "status": paid}
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if StarkInfra.user
        was set before function call
    ## Return:
    - dict of StarkInfra objects with updated attributes
    """
    return rest.get_raw(
        path=path,
        query=query,
        user=user,
        prefix=request_methods_prefix,
        raiseException=False
    )


def post(path, body=None, query=None, user=None):
    """# Create any StarkInfra resource
    Send a list of jsons and create any StarkInfra resource objects
    ## Parameters (required):
    - path [string]: StarkInfra resource's route. ex: "/pix-request/"
    - body [dict]: request parameters. ex: {"requests": [{"amount": 100, "name": "Iron Bank S.A.", "taxId": "20.018.183/0001-80"}]}
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if StarkInfra.user
        was set before function call
    - query [dict, default None]: Query parameters. ex: {"limit": 1, "status": paid}
    ## Return:
    - list of resources jsons with updated attributes
    """
    return rest.post_raw(
        path=path,
        payload=body,
        query=query,
        user=user,
        prefix=request_methods_prefix,
        raiseException=False
    )


def patch(path, body=None, user=None):
    """# Update any StarkInfra resource
    Send a json with parameters of a single StarkInfra resource object and update it
    ## Parameters (required):
    - path [string]: StarkInfra resource's route. ex: "/pix-request/5699165527090460"
    - body [dict]: request parameters. ex: {"amount": 100}
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if StarkInfra.user
        was set before function call
    ## Return:
    - json of the resource with updated attributes
    """
    return rest.patch_raw(
        path=path,
        payload=body,
        user=user,
        prefix=request_methods_prefix,
        raiseException=False
    )


def put(path, body=None, user=None):
    """# Put any StarkInfra resource
        Send a json with parameters of a single StarkInfra resource object and create it, if the resource alredy exists,
        you will update it.
        ## Parameters (required):
        - path [string]: StarkInfra resource's route. ex: "/pix-request"
        - body [dict]: request parameters. ex: {"amount": 100}
        ## Parameters (optional):
        - user [Organization/Project object, default None]: Organization or Project object. Not necessary if StarkInfra.user
            was set before function call
        ## Return:
        - json of the resource with updated attributes
        """
    return rest.put_raw(
        path=path,
        payload=body,
        user=user,
        prefix=request_methods_prefix,
        raiseException=False
    )


def delete(path, body=None, user=None):
    """# Delete any StarkInfra resource
        Send a json with parameters of a single StarkInfra resource object and delete it
        you will update it.
        ## Parameters (required):
        - path [string]: StarkInfra resource's route. ex: "/pix-request/5699165527090460"
        - body [dict]: request parameters. ex: {"amount": 100}
        ## Parameters (optional):
        - user [Organization/Project object, default None]: Organization or Project object. Not necessary if StarkInfra.user
            was set before function call
        ## Return:
        - json of the resource with updated attributes
        """
    return rest.delete_raw(
        path=path,
        payload=body,
        user=user,
        prefix=request_methods_prefix,
        raiseException=False
    )



from starkcore.utils.checks import check_datetime, check_date
from ..__pixkey import _resource as _pixkey_resource
from starkcore.utils.resource import Resource
from starkcore.utils.api import from_api_json
from ...utils import rest


class Log(Resource):
    """# PixKey.Log object
    Every time a PixKey entity is modified, a corresponding PixKey.Log
    is generated for the entity. This log is never generated by the user.
    ## Attributes (return-only):
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - key [PixKey]: PixKey entity to which the log refers to.
    - type [string]: type of the PixKey event which triggered the log creation. ex: "created", "registered", "updated", "failed", "canceling", "canceled"
    - errors [list of strings]: list of errors linked to this PixKey event
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """
    def __init__(self, id, key, type, errors, created):
        Resource.__init__(self, id=id)

        self.key = from_api_json(_pixkey_resource, key)
        self.type = type
        self.errors = errors
        self.created = check_datetime(created)


_resource = {"class": Log, "name": "PixKeyLog"}


def get(id, user=None):
    """# Retrieve a specific PixKey.Log
    Receive a single PixKey.Log object previously created by the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixKey.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(ids=None, limit=None, after=None, before=None, types=None, key_ids=None, user=None):
    """# Retrieve PixKey.Logs
    Receive a generator of PixKey.Log objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter retrieved objects by types. ex: ["created", "registered", "updated", "failed", "canceling", "canceled"]
    - key_ids [list of strings, default None]: list of PixKey IDs to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: Log ids to filter PixKey Logs. ex: ["5656565656565656"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of PixKey.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        key_ids=key_ids,
        user=user,
    )


def page(cursor=None, ids=None, limit=None, after=None, before=None, types=None, key_ids=None, user=None):
    """# Retrieve paged PixKey.Logs
    Receive a list of up to 100 PixKey.Log objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your keys.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - ids [list of strings, default None]: Log ids to filter PixKey Logs. ex: ["5656565656565656"]
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter retrieved objects by types. ex: ["created", "registered", "updated", "failed", "canceling", "canceled"]
    - key_ids [list of strings, default None]: list of PixKey IDs to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of PixKey.Log objects with updated attributes
    - cursor to retrieve the next page of PixKey.Log objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        key_ids=key_ids,
        user=user,
    )



from ...utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.api import from_api_json
from starkcore.utils.checks import check_datetime, check_date
from ..__pixclaim import _resource as _pixclaim_resource


class Log(Resource):
    """# pixclaim.Log object
    Every time a PixClaim entity is modified, a corresponding PixClaim.Log
    is generated for the entity. This log is never generated by the user.
    ## Attributes (return-only):
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - claim [PixClaim]: PixClaim entity to which the log refers to.
    - type [string]: type of the PixClaim event which triggered the log creation. ex: "created", "failed", "delivering", "delivered", "confirming", "confirmed", "success", "canceling", "canceled"
    - errors [list of strings]: list of errors linked to this PixClaim event
    - reason [string]: reason why the PixClaim was modified, resulting in the Log. Options: "fraud", "userRequested", "accountClosure", "defaultOperation", "reconciliation"
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """
    
    def __init__(self, id, claim, type, errors, reason, created):
        Resource.__init__(self, id=id)

        self.claim = from_api_json(_pixclaim_resource, claim)
        self.type = type
        self.errors = errors
        self.reason = reason
        self.created = check_datetime(created)


_resource = {"class": Log, "name": "PixClaimLog"}


def get(id, user=None):
    """# Retrieve a specific PixClaim.Log
    Receive a single PixClaim.Log object previously created by the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixClaim.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(ids=None, limit=None, after=None, before=None, types=None, claim_ids=None, user=None):
    """# Retrieve PixClaim.Logs
    Receive a generator of PixClaim.Log objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter retrieved objects by types. ex: ["created", "failed", "delivering", "delivered", "confirming", "confirmed", "success", "canceling", "canceled"]
    - claim_ids [list of strings, default None]: list of PixClaim ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: Log ids to filter PixClaim Logs. ex: ["5656565656565656"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of PixClaim.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        claim_ids=claim_ids,
        user=user,
    )


def page(cursor=None, ids=None, limit=None, after=None, before=None, types=None, claim_ids=None, user=None):
    """# Retrieve paged PixClaim.Logs
    Receive a list of up to 100 PixClaim.Log objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your claims.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter retrieved objects by types. ex: ["created", "failed", "delivering", "delivered", "confirming", "confirmed", "success", "canceling", "canceled"]
    - claim_ids [list of strings, default None]: list of PixClaim IDs to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: Log ids to filter PixClaim Logs. ex: ["5656565656565656"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of PixClaim.Log objects with updated attributes
    - cursor to retrieve the next page of PixClaim.Log objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        claim_ids=claim_ids,
        user=user,
    )



from ...utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class Attempt(Resource):
    """# Event.Attempt object
    When an Event delivery fails, an event attempt will be registered.
    It carries information meant to help you debug event reception issues.
    ## Attributes (return-only):
    - id [string]: unique id that identifies the delivery attempt. ex: "5656565656565656"
    - code [string]: delivery error code. ex: badHttpStatus, badConnection, timeout
    - message [string]: delivery error full description. ex: "HTTP POST request returned status 404"
    - event_id [string]: ID of the Event whose delivery failed. ex: "4848484848484848"
    - webhook_id [string]: ID of the Webhook that triggered this event. ex: "5656565656565656"
    - created [datetime.datetime]: datetime representing the moment when the attempt was made. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, code, message, event_id, webhook_id, created):
        Resource.__init__(self, id=id)

        self.code = code
        self.message = message
        self.webhook_id = webhook_id
        self.event_id = event_id
        self.created = check_datetime(created)


_resource = {"class": Attempt, "name": "EventAttempt"}


def get(id, user=None):
    """# Retrieve a specific event.Attempt
    Receive a single event.Attempt object previously created by the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - event.Attempt object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, event_ids=None, webhook_ids=None, user=None):
    """# Retrieve event.Attempts
    Receive a generator of event.Attempt objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - event_ids [list of strings, default None]: list of Event ids to filter attempts. ex: ["5656565656565656", "4545454545454545"]
    - webhook_ids [list of strings, default None]: list of Webhook ids to filter attempts. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of event.Attempt objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        event_ids=event_ids,
        webhook_ids=webhook_ids,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, event_ids=None, webhook_ids=None, user=None):
    """# Retrieve paged event.Attempts
    Receive a list of up to 100 event.Attempt objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - event_ids [list of strings, default None]: list of Event ids to filter attempts. ex: ["5656565656565656", "4545454545454545"]
    - webhook_ids [list of strings, default None]: list of Webhook ids to filter attempts. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of event.Attempt objects with updated attributes
    - cursor to retrieve the next page of event.Attempt objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        event_ids=event_ids,
        webhook_ids=webhook_ids,
        user=user,
    )



from ..utils import rest
from starkcore.utils.resource import Resource


class Webhook(Resource):
    """# Webhook subscription object
    A Webhook is used to subscribe to notification events on a user-selected endpoint.
    Currently, available services for subscription are credit-note, issuing-card, issuing-invoice, issuing-purchase, pix-request.in, pix-request.out, pix-reversal.in, pix-reversal.out, pix-claim, pix-key, pix-chargeback, pix-infraction,
    ## Parameters (required):
    - url [string]: Url that will be notified when an event occurs.
    - subscriptions [list of strings]: list of any non-empty combination of the available services. ex: ["contract", "credit-note", "signer", "issuing-card", "issuing-invoice", "issuing-purchase", "pix-request.in", "pix-request.out", "pix-reversal.in", "pix-reversal.out", "pix-claim", "pix-key", "pix-chargeback", "pix-infraction"]
    ## Attributes (return-only):
    - id [string]: unique id returned when the webhook is created. ex: "5656565656565656"
    """

    def __init__(self, url, subscriptions, id=None):
        Resource.__init__(self, id=id)

        self.url = url
        self.subscriptions = subscriptions


_resource = {"class": Webhook, "name": "Webhook"}


def create(url, subscriptions, user=None):
    """# Create Webhook subscription
    Send a single Webhook subscription for creation at the Stark Infra API
    ## Parameters (required):
    - url [string]: url to which notification events will be sent to. ex: "https://webhook.site/60e9c18e-4b5c-4369-bda1-ab5fcd8e1b29"
    - subscriptions [list of strings]: list of any non-empty combination of the available services. ex: ["contract", "credit-note", "signer"]
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - Webhook object with updated attributes
    """
    return rest.post_single(resource=_resource, entity=Webhook(url=url, subscriptions=subscriptions), user=user)


def get(id, user=None):
    """# Retrieve a specific Webhook subscription
    Receive a single Webhook subscription object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - Webhook object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, user=None):
    """# Retrieve Webhook subscriptions
    Receive a generator of Webhook subscription objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of Webhook objects with updated attributes
    """
    return rest.get_stream(resource=_resource, limit=limit, user=user)


def page(cursor=None, limit=None, user=None):
    """# Retrieve paged Webhooks subscriptions
    Receive a list of up to 100 Webhook subscription objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of Webhook objects with updated attributes
    - cursor to retrieve the next page of Webhook objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        user=user,
    )


def delete(id, user=None):
    """# Delete a Webhook subscriptions entity
    Delete a Webhook subscriptions entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: Webhook unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - deleted Webhook object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from starkcore.utils.subresource import SubResource
from starkinfra.utils import rest


class CardMethod(SubResource):
    """# CardMethod object
    CardMethod's codes are used to define methods filters in IssuingRules.
    ## Parameters (required):
    - code [string]: method's code. Options: "chip", "token", "server", "manual", "magstripe", "contactless"
    ## Attributes (return-only):
    - name [string]: method's name. ex: "token"
    - number [string]: method's number. ex: "81"
    """

    def __init__(self, code, name=None, number=None):
        self.code = code
        self.name = name
        self.number = number


_resource = {"class": CardMethod, "name": "CardMethod"}


def query(search=None, user=None):
    """# Retrieve CardMethods
    Receive a generator of CardMethod objects available in the Stark Infra API
    ## Parameters (optional):
    - search [string, default None]: keyword to search for code, name or number. ex:"token"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of CardMethod objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        search=search,
        user=user,
    )



version = "0.21.0"
language = "en-US"
timeout = 15
user = None
request_methods_prefix = "Joker"

from starkcore import Project, Organization, key, error

from . import event
from .event.__event import Event

from . import brcodepreview
from .brcodepreview.__brcodepreview import BrcodePreview

from . import pixrequest
from .pixrequest.__pixrequest import PixRequest

from . import pixreversal
from .pixreversal.__pixreversal import PixReversal

from . import pixstatement
from .pixstatement.__pixstatement import PixStatement

from . import pixbalance
from .pixbalance.__pixbalance import PixBalance

from . import pixdirector
from .pixdirector.__pixdirector import PixDirector

from . import pixkey
from .pixkey.__pixkey import PixKey

from . import pixclaim
from .pixclaim.__pixclaim import PixClaim

from . import pixdomain
from .pixdomain.__pixdomain import PixDomain

from . import pixfraud
from .pixfraud.__pixfraud import PixFraud

from . import pixinfraction
from .pixinfraction.__pixinfraction import PixInfraction

from . import pixchargeback
from .pixchargeback.__pixchargeback import PixChargeback

from . import pixuser
from .pixuser.__pixuser import PixUser

from . import issuingbalance
from .issuingbalance.__issuingbalance import IssuingBalance

from . import issuingbillinginvoice
from .issuingbillinginvoice.__issuingbillinginvoice import IssuingBillingInvoice

from . import issuingbillingtransaction
from .issuingbillingtransaction.__issuingbillingtransaction import IssuingBillingTransaction

from . import creditnote
from .creditnote.__creditnote import CreditNote

from . import creditsigner
from .creditsigner.__creditsigner import CreditSigner

from . import creditpreview
from .creditpreview.__creditpreview import CreditPreview

from . import creditholmes
from .creditholmes.__creditholmes import CreditHolmes

from . import individualidentity
from .individualidentity.__individualidentity import IndividualIdentity

from . import individualdocument
from .individualdocument.__individualdocument import IndividualDocument

from . import dynamicbrcode
from .dynamicbrcode.__dynamicbrcode import DynamicBrcode

from . import staticbrcode
from .staticbrcode.__staticbrcode import StaticBrcode

from . import issuingtransaction
from .issuingtransaction.__issuingtransaction import IssuingTransaction

from . import issuingholder
from .issuingholder.__issuingholder import IssuingHolder

from . import issuingcard
from .issuingcard.__issuingcard import IssuingCard

from . import issuingpurchase
from .issuingpurchase.__issuingpurchase import IssuingPurchase

from . import issuinginvoice
from .issuinginvoice.__issuinginvoice import IssuingInvoice

from . import issuingwithdrawal
from .issuingwithdrawal.__issuingwithdrawal import IssuingWithdrawal

from . import issuingproduct
from .issuingproduct.__issuingproduct import IssuingProduct

from . import issuingrule
from .issuingrule.__issuingrule import IssuingRule

from . import issuingstock
from .issuingstock.__issuingstock import IssuingStock

from . import issuingrestock
from .issuingrestock.__issuingrestock import IssuingRestock

from . import issuingdesign
from .issuingdesign.__issuingdesign import IssuingDesign

from . import issuingembossingrequest
from .issuingembossingrequest.__issuingembossingrequest import IssuingEmbossingRequest

from . import issuingembossingkit
from .issuingembossingkit.__issuingembossingkit import IssuingEmbossingKit

from . import issuingtoken
from .issuingtoken.__issuingtoken import IssuingToken

from . import issuingtokendesign
from .issuingtokendesign.__issuingtokendesign import IssuingTokenDesign

from . import issuingtokenrequest
from .issuingtokenrequest.__issuingtokenrequest import IssuingTokenRequest

from . import issuingtokenactivation
from .issuingtokenactivation.__issuingtokenactivation import IssuingTokenActivation

from . import merchantcategory
from .merchantcategory.__merchantcategory import MerchantCategory

from . import merchantcountry
from .merchantcountry.__merchantcountry import MerchantCountry

from . import cardmethod
from .cardmethod.__cardmethod import CardMethod

from . import webhook
from .webhook.__webhook import Webhook

from .utils import endtoendid, returnid

from . import request


from json import dumps
from ..utils import rest
from ..utils.parse import parse_and_verify
from starkcore.utils.api import api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class IssuingPurchase(Resource):
    """# IssuingPurchase object
    Displays the IssuingPurchase objects created in your Workspace.
    ## Attributes (return-only):
    - holder_name [string]: card holder name. ex: "Tony Stark"
    - product_id [string]: unique card product number (BIN) registered within the card network. ex: "53810200"
    - card_id [string]: unique id returned when IssuingCard is created. ex: "5656565656565656"
    - card_ending [string]: last 4 digits of the card number. ex: "1234"
    - purpose [string]: purchase purpose. ex: "purchase"
    - installment_count [integer]: quantity of installments to be confirmed. Minimum = 1. ex: 12
    - amount [integer]: IssuingPurchase value in cents. Minimum = 0. ex: 1234 (= R$ 12.34)
    - tax [integer]: IOF amount taxed for international purchases. ex: 1234 (= R$ 12.34)
    - issuer_amount [integer]: issuer amount. ex: 1234 (= R$ 12.34)
    - issuer_currency_code [string]: issuer currency code. ex: "USD"
    - issuer_currency_symbol [string]: issuer currency symbol. ex: "$"
    - merchant_amount [integer]: merchant amount. ex: 1234 (= R$ 12.34)
    - merchant_currency_code [string]: merchant currency code. ex: "USD"
    - merchant_currency_symbol [string]: merchant currency symbol. ex: "$"
    - merchant_category_code [string]: merchant category code. ex: "fastFoodRestaurants"
    - merchant_category_type [string]: merchant category type. ex "food"
    - merchant_country_code [string]: merchant country code. ex: "USA"
    - acquirer_id [string]: acquirer ID. ex: "5656565656565656"
    - merchant_id [string]: merchant ID. ex: "5656565656565656"
    - merchant_name [string]: merchant name. ex: "Google Cloud Platform"
    - merchant_fee [integer]: fee charged by the merchant to cover specific costs, such as ATM withdrawal logistics, etc. ex: 200 (= R$ 2.00)
    - wallet_id [string]: virtual wallet ID. ex: "5656565656565656"
    - method_code [string]: method code. Options: "chip", "token", "server", "manual", "magstripe" or "contactless"
    - score [float]: internal score calculated for the authenticity of the purchase. None in case of insufficient data. ex: 7.6
    - end_to_end_id [string]: Unique id used to identify the transaction through all of its life cycle, even before the purchase is denied or approved and gets its usual id. ex: "679cd385-642b-49d0-96b7-89491e1249a5"
    - tags [list of strings]: list of strings for tagging returned by the sub-issuer during the authorization. ex: ["travel", "food"]
    ## Attributes (IssuingPurchase only):
    - id [string]: unique id returned when IssuingPurchase is created. ex: "5656565656565656"
    - issuing_transaction_ids [list of strings]: ledger transaction ids linked to this Purchase
    - status [string]: current IssuingCard status. Options: "approved", "canceled", "denied", "confirmed", "voided"
    - description [string]: IssuingPurchase description. ex: "Office Supplies"
    - metadata [dictionary]: dictionary object used to store additional information about the IssuingPurchase object. ex: { authorizationId: "OjZAqj" }.
    - zip_code [string]: zip code of the merchant location. ex: "02101234"
    - updated [datetime.datetime]: latest update datetime for the IssuingPurchase. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the IssuingPurchase. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    ## Attributes (authorization request only):
    - is_partial_allowed [bool]: true if the merchant allows partial purchases. ex: False
    - card_tags [list of strings]: tags of the IssuingCard responsible for this purchase. ex: ["travel", "food"]
    - holder_id [string]: card holder ID. ex: "5656565656565656"
    - holder_tags [list of strings]: tags of the IssuingHolder responsible for this purchase. ex: ["technology", "john snow"]
    """

    def __init__(self, holder_name=None, product_id=None, card_id=None, card_ending=None, purpose=None,
                 installment_count=None, amount=None, tax=None, issuer_amount=None, issuer_currency_code=None,
                 issuer_currency_symbol=None, merchant_amount=None, merchant_currency_code=None,
                 merchant_currency_symbol=None, merchant_category_code=None, merchant_category_type=None,
                 merchant_country_code=None, acquirer_id=None, merchant_id=None, merchant_name=None, merchant_fee=None,
                 wallet_id=None, method_code=None, score=None, end_to_end_id=None, tags=None, id=None,
                 issuing_transaction_ids=None, status=None, description=None, metadata=None, zip_code=None,
                 updated=None, created=None, is_partial_allowed=None, card_tags=None, holder_id=None, holder_tags=None):
        Resource.__init__(self, id=id)

        self.holder_name = holder_name
        self.product_id = product_id
        self.card_id = card_id
        self.card_ending = card_ending
        self.purpose = purpose
        self.installment_count = installment_count
        self.amount = amount
        self.tax = tax
        self.issuer_amount = issuer_amount
        self.issuer_currency_code = issuer_currency_code
        self.issuer_currency_symbol = issuer_currency_symbol
        self.merchant_amount = merchant_amount
        self.merchant_currency_code = merchant_currency_code
        self.merchant_currency_symbol = merchant_currency_symbol
        self.merchant_category_code = merchant_category_code
        self.merchant_category_type = merchant_category_type
        self.merchant_country_code = merchant_country_code
        self.acquirer_id = acquirer_id
        self.merchant_id = merchant_id
        self.merchant_name = merchant_name
        self.merchant_fee = merchant_fee
        self.wallet_id = wallet_id
        self.method_code = method_code
        self.score = score
        self.end_to_end_id = end_to_end_id
        self.tags = tags
        self.issuing_transaction_ids = issuing_transaction_ids
        self.status = status
        self.description = description
        self.metadata = metadata
        self.zip_code = zip_code
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)
        self.is_partial_allowed = is_partial_allowed
        self.card_tags = card_tags
        self.holder_id = holder_id
        self.holder_tags = holder_tags


_resource = {"class": IssuingPurchase, "name": "IssuingPurchase"}


def get(id, user=None):
    """# Retrieve a specific IssuingPurchase
    Receive a single IssuingPurchase object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - IssuingPurchase object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(ids=None, limit=None, after=None, before=None, end_to_end_ids=None, holder_ids=None, card_ids=None,
          status=None, user=None):
    """# Retrieve IssuingPurchase
    Receive a generator of IssuingPurchase objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - end_to_end_ids [list of strings, default None]: central bank's unique transaction ID. ex: "E79457883202101262140HHX553UPqeq"
    - holder_ids [list of strings, default None]: card holder IDs. ex: ["5656565656565656", "4545454545454545"]
    - card_ids [list of strings, default None]: card  IDs. ex: ["5656565656565656", "4545454545454545"]
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["approved", "canceled", "denied", "confirmed", "voided"]
    - ids [list of strings, default None, default None]: purchase IDs
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of IssuingPurchase objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        end_to_end_ids=end_to_end_ids,
        holder_ids=holder_ids,
        card_ids=card_ids,
        status=status,
        user=user,
    )


def page(end_to_end_ids=None, holder_ids=None, card_ids=None, status=None, after=None, before=None, ids=None,
        cursor=None, limit=None, user=None):
    """# Retrieve paged IssuingPurchases
    Receive a list of IssuingPurchase objects previously created in the Stark Infra API and the cursor to the next page.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - end_to_end_ids [list of strings, default None]: central bank's unique transaction ID. ex: "E79457883202101262140HHX553UPqeq"
    - holder_ids [list of strings, default None]: card holder IDs. ex: ["5656565656565656", "4545454545454545"]
    - card_ids [list of strings, default None]: card  IDs. ex: ["5656565656565656", "4545454545454545"]
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["approved", "canceled", "denied", "confirmed", "voided"]
    - ids [list of strings, default None]: purchase IDs
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of IssuingPurchase objects with updated attributes
    - cursor to retrieve the next page of IssuingPurchase objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        end_to_end_ids=end_to_end_ids,
        holder_ids=holder_ids,
        card_ids=card_ids,
        status=status,
        user=user,
    )


def update(id, description=None, tags=None, user=None):
    """# Update IssuingPurchase entity
    Update an IssuingPurchase by passing id.
    ## Parameters (required):
    - id [string]: IssuingPurchase id. ex: '5656565656565656'
    ## Parameters (optional):
    - tags [list of strings]: list of strings for tagging. ex: ["tony", "stark"]
    - description [string]: IssuingPurchase description. ex: "Office Supplies"
    ## Return:
    - target IssuingPurchase with updated attributes
    """
    payload = {
        "tags": tags,
        "description": description
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def parse(content, signature, user=None):
    """# Create a single verified IssuingPurchase authorization request from a content string
    Use this method to parse and verify the authenticity of the authorization request received at the informed endpoint.
    Authorization requests are posted to your registered endpoint whenever IssuingPurchases are received.
    They present IssuingPurchase data that must be analyzed and answered with approval or declination.
    If the provided digital signature does not check out with the StarkInfra public key, a stark.exception.InvalidSignatureException will be raised.
    If the authorization request is not answered within 2 seconds or is not answered with an HTTP status code 200 the IssuingPurchase will go through the pre-configured stand-in validation.
    ## Parameters (required):
    - content [string]: response content from request received at user endpoint (not parsed)
    - signature [string]: base-64 digital signature received at response header "Digital-Signature"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - Parsed IssuingPurchase object
    """
    return parse_and_verify(
        content=content,
        signature=signature,
        user=user,
        resource=_resource,
        key="",
    )


def response(status, amount=None, reason=None, tags=None):
    """# Helps you respond IssuingPurchase requests
    ## Parameters (required):
    - status [string]: sub-issuer response to the authorization. ex: "approved" or "denied"
    ## Parameters (conditionally required):
    - reason [string]: denial reason. Options: "other", "blocked", "lostCard", "stolenCard", "invalidPin", "invalidCard", "cardExpired", "issuerError", "concurrency", "standInDenial", "subIssuerError", "invalidPurpose", "invalidZipCode", "invalidWalletId", "inconsistentCard", "settlementFailed", "cardRuleMismatch", "invalidExpiration", "prepaidInstallment", "holderRuleMismatch", "insufficientBalance", "tooManyTransactions", "invalidSecurityCode", "invalidPaymentMethod", "confirmationDeadline", "withdrawalAmountLimit", "insufficientCardLimit", "insufficientHolderLimit"
    ## Parameters (optional):
    - amount [integer, default None]: amount in cents that was authorized. ex: 1234 (= R$ 12.34)
    - tags [list of strings, default None]: tags to filter retrieved object. ex: ["tony", "stark"]
    ## Return:
    - Dumped JSON string that must be returned to us on the IssuingPurchase request
    """
    params = {"authorization": {
        "status": status,
        "amount": amount,
        "reason": reason,
        "tags": tags,
    }}
    return dumps(api_json(params))



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_datetime_or_date


class Transfer(Resource):
    """# creditnote.Transfer object
    Transfer sent to the credit receiver after the contract is signed.
    ## Parameters (required):
    - name [string]: receiver full name. ex: "Anthony Edward Stark"
    - tax_id [string]: receiver tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - bank_code [string]: code of the receiver bank institution in Brazil. ex: "20018183" or "341"
    - branch_code [string]: receiver bank account branch. Use '-' in case there is a verifier digit. ex: "1357-9"
    - account_number [string]: receiver bank account number. Use '-' before the verifier digit. ex: "876543-2"
    ## Parameters (optional):
    - account_type [string, default "checking"]: Receiver bank account type. This parameter only has effect on Pix Transfers. ex: "checking", "savings", "salary" or "payment"
    - tags [list of strings, default []]: list of strings for reference when searching for transfers. ex: ["employees", "monthly"]
    ## Attributes (return-only):
    - id [string]: unique id returned when the transfer is created. ex: "5656565656565656"
    - amount [integer]: amount in cents to be transferred. ex: 1234 (= R$ 12.34)
    - external_id [string]: url safe string that must be unique among all your transfers. Duplicated external_ids will cause failures. By default, this parameter will block any transfer that repeats amount and receiver information on the same date. ex: "my-internal-id-123456"
    - scheduled [datetime.date, datetime.datetime or string]: date or datetime when the transfer will be processed. May be pushed to next business day if necessary. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - description [string]: optional description to override default description to be shown in the bank statement. ex: "Payment for service #1234"
    - fee [integer]: fee charged when the Transfer is processed. ex: 200 (= R$ 2.00)
    - status [string]: current transfer status. ex: "success" or "failed"
    - transaction_ids [list of strings]: ledger Transaction IDs linked to this Transfer (if there are two, the second is the chargeback). ex: ["19827356981273"]
    - created [datetime.datetime]: creation datetime for the transfer. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the transfer. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, name, tax_id, bank_code, branch_code, account_number, account_type=None, tags=None, id=None,
                 amount=None, external_id=None, scheduled=None, description=None, fee=None, status=None,
                 transaction_ids=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.name = name
        self.tax_id = tax_id
        self.bank_code = bank_code
        self.branch_code = branch_code
        self.account_number = account_number
        self.account_type = account_type
        self.tags = tags
        self.amount = amount
        self.external_id = external_id
        self.scheduled = check_datetime_or_date(scheduled)
        self.description = description
        self.fee = fee
        self.status = status
        self.transaction_ids = transaction_ids
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": Transfer, "name": "Transfer"}



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_date, check_datetime
from ..utils import rest


class IssuingStock(Resource):
    """# IssuingStock object
    The IssuingStock object represents the current stock of a certain IssuingDesign linked to an Embosser available to your workspace.
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingStock is created. ex: "5656565656565656"
    - balance [integer]: [EXPANDABLE] current stock balance. ex: 1000
    - design_id [string]: IssuingDesign unique id. ex: "5656565656565656"
    - embosser_id [string]: Embosser unique id. ex: "5656565656565656"
    - updated [datetime.datetime]: latest update datetime for the IssuingStock. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the IssuingStock. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, balance=None, design_id=None, embosser_id=None, id=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.balance = balance
        self.design_id = design_id
        self.embosser_id = embosser_id
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": IssuingStock, "name": "IssuingStock"}


def query(limit=None, after=None, before=None, design_ids=None, embosser_ids=None, ids=None,
          expand=None, user=None):
    """# Retrieve IssuingStocks
    Receive a generator of IssuingStock objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - design_ids [list of strings, default None]: IssuingDesign unique ids. ex: ["5656565656565656", "4545454545454545"]
    - embosser_ids [list of strings, default None]: Embosser unique ids. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - expand [list of strings, default None]: fields to expand information. ex: ["balance"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of IssuingStock objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        design_ids=design_ids,
        embosser_ids=embosser_ids,
        ids=ids,
        expand=expand,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, design_ids=None, embosser_ids=None, 
         ids=None, expand=None, user=None):
    """# Retrieve paged IssuingStocks
    Receive a list of up to 100 IssuingStock objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - design_ids [list of strings, default None]: IssuingDesign unique ids. ex: ["5656565656565656", "4545454545454545"]
    - embosser_ids [list of strings, default None]: Embosser unique ids. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - expand [list of strings, default None]: fields to expand information. ex: ["balance"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IssuingStock objects with updated attributes
    - cursor to retrieve the next page of IssuingStock objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        design_ids=design_ids,
        embosser_ids=embosser_ids,
        ids=ids,
        expand=expand,
        user=user,
    )


def get(id, expand=None, user=None):
    """# Retrieve a specific IssuingStock
    Receive a single IssuingStock object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - expand [list of strings, default None]: fields to expand information. ex: ["balance"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingStock object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, expand=expand, user=user)



from starkcore.utils.api import from_api_json
from starkcore.utils.subresource import SubResource
from ..creditnote.invoice.__invoice import Invoice, _resource as _invoice_resource


class CreditNotePreview(SubResource):
    """# CreditNotePreview object
    A CreditNotePreview is used to preview a CCB contract between the borrower and lender with a specific table type.
    When you initialize a CreditNotePreview, the entity will be automatically sent to the Stark Infra API.
    The 'create' function sends the objects
    to the Stark Infra API and returns the list of preview data.
    ## Parameters (required):
    - type [string]: table type that defines the amortization system. Options: "sac", "price", "american", "bullet", "custom"
    - nominal_amount [integer]: amount in cents transferred to the credit receiver, before deductions. ex: 11234 (= R$ 112.34)
    - scheduled [datetime.date, datetime.datetime or string]: date of transfer execution. ex: datetime(2020, 3, 10)
    - tax_id [string]: credit receiver's tax ID (CPF or CNPJ). ex: "20.018.183/0001-80"
    ## Parameters (conditionally required):
    - invoices [list of Invoice objects]: list of Invoice objects to be created and sent to the credit receiver. ex: [Invoice(), Invoice()]
    - nominal_interest [float]: yearly nominal interest rate of the credit note, in percentage. ex: 12.5
    - initial_due [datetime.date, datetime.datetime or string]: date of the first invoice. ex: datetime(2020, 3, 10)
    - count [integer]: quantity of invoices for payment. ex: 12
    - initial_amount [integer]: value of the first invoice in cents. ex: 1234 (= R$12.34)
    - interval [string]: interval between invoices. ex: "year", "month"
    ## Parameters (optional):
    - rebate_amount [integer, default None]: credit analysis fee deducted from lent amount. ex: 11234 (= R$ 112.34)
    ## Attributes (return-only):
    - amount [integer]: CreditNote value in cents. ex: 1234 (= R$ 12.34)
    - interest [float]: yearly effective interest rate of the credit note, in percentage. ex: 12.5
    - tax_amount [integer]: tax amount included in the CreditNote. ex: 100
    """

    def __init__(self, type, nominal_amount, scheduled, tax_id, invoices=None, nominal_interest=None,
                 initial_due=None, count=None, initial_amount=None, interval=None, rebate_amount=None,
                 amount=None, interest=None, tax_amount=None):
        self.type = type
        self.nominal_amount = nominal_amount
        self.scheduled = scheduled
        self.tax_id = tax_id
        self.invoices = _parse_optional_invoices(invoices)
        self.nominal_interest = nominal_interest
        self.initial_due = initial_due
        self.count = count
        self.initial_amount = initial_amount
        self.interval = interval
        self.rebate_amount = rebate_amount
        self.amount = amount
        self.interest = interest
        self.tax_amount = tax_amount


_sub_resource = {"class": CreditNotePreview, "name": "CreditNotePreview"}


def _parse_optional_invoices(invoices):
    if invoices is None:
        return None

    parsed_invoices = []
    for invoice in invoices:
        if isinstance(invoice, Invoice):
            parsed_invoices.append(invoice)
            continue
        parsed_invoices.append(from_api_json(_invoice_resource, invoice))
    return parsed_invoices



from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ...utils import rest
from ..__issuingholder import _resource as _issuing_holder_resource


class Log(Resource):
    """# issuingholder.Log object
    Every time an IssuingHolder entity is updated, a corresponding issuingholder.Log
    is generated for the entity. This log is never generated by the
    user, but it can be retrieved to check additional information
    on the IssuingHolder.
    ## Attributes (return-only):
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - holder [IssuingHolder]: IssuingHolder entity to which the log refers to.
    - type [string]: type of the IssuingHolder event which triggered the log creation. ex: "blocked", "canceled", "created", "unblocked", "updated"
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, holder, type, created):
        Resource.__init__(self, id=id)

        self.holder = from_api_json(_issuing_holder_resource, holder)
        self.type = type
        self.created = check_datetime(created)


_resource = {"class": Log, "name": "IssuingHolderLog"}


def get(id, user=None):
    """# Retrieve a specific issuingholder.Log
    Receive a single issuingholder.Log object previously created by the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - issuingholder.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(ids=None, limit=None, after=None, before=None, types=None, holder_ids=None, user=None):
    """# Retrieve issuingholder.Log
    Receive a generator of issuingholder.Log objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: ["created", "blocked"]
    - holder_ids [list of strings, default None]: list of IssuingHolder ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of issuingholder.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        holder_ids=holder_ids,
        user=user,
    )


def page(cursor=None, ids=None, limit=None, after=None, before=None, types=None, holder_ids=None, user=None):
    """# Retrieve paged issuingholder.Log
    Receive a list of up to 100 issuingholder.Log objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: ["created", "blocked"]
    - holder_ids [list of strings, default None]: list of IssuingHolder ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of issuingholder.Log objects with updated attributes
    - cursor to retrieve the next page of issuingholder.Log objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        holder_ids=holder_ids,
        user=user,
    )



from . import log
from .log.__log import Log
from .invoice.__invoice import Invoice
from .__transfer import Transfer
from . import invoice
from .invoice.__invoice import Invoice
from .invoice.__discount import Discount
from .invoice.__description import Description
from .__creditnote import create, get, query, page, cancel, pdf



import starkinfra
from starkcore.utils.host import StarkHost


_api_version = "v2"


def set_relay(func):
    def wrapper(*args, **kwargs):
        kwargs.update({
            "sdk_version": starkinfra.version,
            "host": StarkHost.infra,
            "api_version": kwargs.get("version") or _api_version,
            "user": kwargs.get("user") or starkinfra.user,
            "language": kwargs.get("language") or starkinfra.language,
            "timeout": kwargs.get("timeout") or starkinfra.timeout,
        })
        return func(*args, **kwargs)
    return wrapper



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..utils import rest


class IssuingTransaction(Resource):
    """# IssuingTransaction object
    The IssuingTransaction object created in your Workspace to represent each balance shift.
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingTransaction is created. ex: "5656565656565656"
    - amount [integer]: IssuingTransaction value in cents. ex: 1234 (= R$ 12.34)
    - balance [integer]: balance amount of the Workspace at the instant of the Transaction in cents. ex: 200 (= R$ 2.00)
    - description [string]: IssuingTransaction description. ex: "Buying food"
    - source [string]: source of the transaction. ex: "issuing-purchase/5656565656565656"
    - tags [string]: list of strings inherited from the source resource. ex: ["tony", "stark"]
    - created [datetime.datetime]: creation datetime for the IssuingTransaction. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, amount, balance, description, source, tags, created):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.balance = balance
        self.description = description
        self.source = source
        self.tags = tags
        self.created = created


_resource = {"class": IssuingTransaction, "name": "IssuingTransaction"}


def get(id, user=None):
    """# Retrieve a specific IssuingTransaction
    Receive a single IssuingTransaction object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingTransaction object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(source=None, tags=None, external_ids=None, after=None, before=None,
          ids=None, limit=None, user=None):
    """# Retrieve IssuingTransactions
    Receive a generator of IssuingTransaction objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default 100]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - external_ids [list of strings, default None]: external IDs. ex: ["5656565656565656", "4545454545454545"]
    - status [string, default None]: filter for status of retrieved objects. ex: "approved", "canceled", "denied", "confirmed" or "voided"
    - ids [list of strings, default None, default None]: purchase IDs
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of IssuingTransaction objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        source=source,
        tags=tags,
        external_ids=external_ids,
        after=check_date(after),
        before=check_date(before),
        ids=ids,
        limit=limit,
        user=user,
    )


def page(source=None, tags=None, external_ids=None, after=None, before=None,
         ids=None, limit=None, cursor=None, user=None):
    """# Retrieve paged IssuingTransaction
    Receive a list of IssuingTransaction objects previously created in the Stark Infra API and the cursor to the next page.
    ## Parameters (optional):
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - external_ids [list of strings, default None]: external IDs. ex: ["5656565656565656", "4545454545454545"]
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [string, default None]: filter for status of retrieved objects. ex: "approved", "canceled", "denied", "confirmed" or "voided"
    - ids [list of strings,default None]: purchase IDs
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - cursor [string, default None]: cursor returned on the previous page function call
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IssuingTransaction objects with updated attributes
    - cursor to retrieve the next page of IssuingPurchase objects
    """
    return rest.get_page(
        resource=_resource,
        source=source,
        tags=tags,
        external_ids=external_ids,
        after=check_date(after),
        before=check_date(before),
        ids=ids,
        limit=limit,
        cursor=cursor,
        user=user,
    )



from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ...utils import rest
from ..__issuingcard import _resource as _issuing_card_resource


class Log(Resource):
    """# issuingcard.Log object
    Every time an IssuingCard entity is updated, a corresponding issuingcard.Log
    is generated for the entity. This log is never generated by the
    user, but it can be retrieved to check additional information
    on the IssuingCard.
    ## Attributes (return-only):
    - id [string]: unique id returned when the log is created. ex: "5656565656565656"
    - card [IssuingCard]: IssuingCard entity to which the log refers to.
    - type [string]: type of the IssuingCard event which triggered the log creation. ex: "blocked", "canceled", "created", "expired", "unblocked", "updated"
    - created [datetime.datetime]: creation datetime for the log. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id, card, type, created):
        Resource.__init__(self, id=id)

        self.card = from_api_json(_issuing_card_resource, card)
        self.type = type
        self.created = check_datetime(created)


_resource = {"class": Log, "name": "IssuingCardLog"}


def get(id, user=None):
    """# Retrieve a specific issuingcard.Log
    Receive a single issuingcard.Log object previously created by the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - issuingcard.Log object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(ids=None, card_ids=None, types=None, after=None, before=None, limit=None, user=None):
    """# Retrieve issuingcard.Log
    Receive a generator of issuingcard.Log objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: ["blocked", "canceled", "created", "expired", "unblocked", "updated"]
    - card_ids [list of strings, default None]: list of IssuingCard ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of issuingcard.Log objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        card_ids=card_ids,
        user=user,
    )


def page(cursor=None, ids=None, limit=None, after=None, before=None, types=None, card_ids=None, user=None):
    """# Retrieve paged issuingcard.Log
    Receive a list of up to 100 issuingcard.Log objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - types [list of strings, default None]: filter for log event types. ex: ["blocked", "canceled", "created", "expired", "unblocked", "updated"]
    - card_ids [list of strings, default None]: list of IssuingCard ids to filter logs. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of issuingcard.Log objects with updated attributes
    - cursor to retrieve the next page of issuingcard.Log objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        ids=ids,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        types=types,
        card_ids=card_ids,
        user=user,
    )



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime
from ..issuingdesign.__issuingdesign import parse_designs
from ..utils import rest


class IssuingEmbossingKit(Resource):
    """# IssuingEmbossingKit object
    The IssuingEmbossingKit object displays information on the embossing kits available to your Workspace.
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingEmbossingKit is created. ex: "5656565656565656"
    - name [string]: embossing kit name. ex: "stark-plastic-dark-001"
    - designs [list of IssuingDesign objects]: list of IssuingDesign objects. ex: [IssuingDesign(), IssuingDesign()]
    - updated [datetime.datetime]: latest update datetime for the IssuingEmbossingKit. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the IssuingEmbossingKit. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id=None, name=None, designs=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.name = name
        self.designs = parse_designs(designs)
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": IssuingEmbossingKit, "name": "IssuingEmbossingKit"}


def query(limit=None, after=None, before=None, status=None, design_ids=None, ids=None, user=None):
    """# Retrieve IssuingEmbossingKits
    Receive a generator of IssuingEmbossingKit objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "processing", "success", "failed"]
    - design_ids [list of string, default None]: list of design_ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of IssuingEmbossingKit objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=after,
        before=before,
        status=status,
        design_ids=design_ids,
        ids=ids,
        user=user
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, design_ids=None, ids=None, user=None):
    """# Retrieve paged IssuingEmbossingKits
    Receive a list of up to 100 IssuingEmbossingKit objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "processing", "success", "failed"]
    - design_ids [list of string, default None]: list of design_ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IssuingEmbossingKit objects with updated attributes
    - cursor to retrieve the next page of IssuingEmbossingKit objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=after,
        before=before,
        status=status,
        design_ids=design_ids,
        ids=ids,
        user=user
    )


def get(id, user=None):
    """# Retrieve a specific IssuingEmbossingKit
    Receive a single IssuingEmbossingKit object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingEmbossingKit object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)



from starkcore.utils.resource import Resource
from ..utils import rest


class IssuingProduct(Resource):
    """# IssuingProduct object
    The IssuingProduct object displays information of registered card products to your Workspace.
    They represent a group of cards that begin with the same numbers (id) and offer the same product to end customers.
    ## Attributes (return-only):
    - id [string]: unique card product number (BIN) registered within the card network. ex: "53810200"
    - network [string]: card network flag. ex: "mastercard"
    - funding_type [string]: type of funding used for payment. ex: "credit", "debit"
    - holder_type [string]: holder type. ex: "business", "individual"
    - code [string]: internal code from card flag informing the product. ex: "MRW", "MCO", "MWB", "MCS"
    - created [datetime.datetime]: creation datetime for the IssuingProduct. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id=None, network=None, funding_type=None, holder_type=None, code=None, created=None):
        Resource.__init__(self, id=id)

        self.network = network
        self.funding_type = funding_type
        self.holder_type = holder_type
        self.code = code
        self.created = created


_resource = {"class": IssuingProduct, "name": "IssuingProduct"}


def query(limit=None, user=None):
    """# Retrieve IssuingProducts
    Receive a generator of IssuingProduct objects previously registered in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of IssuingProduct objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        user=user,
    )


def page(limit=None, cursor=None, user=None):
    """# Retrieve paged IssuingProducts
    Receive a list of up to 100 IssuingProduct objects previously registered in the Stark Infra API and the cursor to the next page.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of IssuingProduct objects with updated attributes
    - cursor to retrieve the next page of IssuingProduct objects
    """
    return rest.get_page(
        resource=_resource,
        limit=limit,
        cursor=cursor,
        user=user,
    )



from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date, check_datetime_or_date
from ..utils import rest


class IssuingInvoice(Resource):
    """# IssuingInvoice object
    The IssuingInvoice objects created in your Workspace load your Issuing balance when paid.
    ## Parameters (required):
    - amount [integer]: IssuingInvoice value in cents. Minimum = 0 (R$0,00). ex: 1234 (= R$ 12.34)
    ## Parameters (optional):
    - tax_id [string, default sub-issuer tax ID]: payer tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - name [string, default sub-issuer name]: payer name. ex: "Iron Bank S.A."
    - tags [list of strings, default []]: list of strings for tagging. ex: ["travel", "food"]
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingInvoice is created. ex: "5656565656565656"
    - brcode [string]: BR Code for the Invoice payment. ex: "00020101021226930014br.gov.bcb.pix2571brcode-h.development.starkinfra.com/v2/d7f6546e194d4c64a153e8f79f1c41ac5204000053039865802BR5925Stark Bank S.A. - Institu6009Sao Paulo62070503***63042109"
    - due [datetime.datetime or datetime.date or string]: Invoice due and expiration date in UTC ISO format. ex: "2020-10-28T17:59:26.249976+00:00"
    - link [string]: public Invoice webpage URL. ex: "https://starkbank-card-issuer.development.starkbank.com/invoicelink/d7f6546e194d4c64a153e8f79f1c41ac"
    - status [string]: current IssuingInvoice status. ex: "created", "expired", "overdue", "paid"
    - issuing_transaction_id [string]: ledger transaction ids linked to this IssuingInvoice. ex: "issuing-invoice/5656565656565656"
    - updated [datetime.datetime]: latest update datetime for the IssuingInvoice. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - created [datetime.datetime]: creation datetime for the IssuingInvoice. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, tax_id=None, name=None, tags=None, id=None, brcode=None, due=None, link=None, status=None, 
                issuing_transaction_id=None, updated=None, created=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.tax_id = tax_id
        self.name = name
        self.tags = tags
        self.brcode = brcode
        self.due = check_datetime_or_date(due)
        self.link = link
        self.status = status
        self.issuing_transaction_id = issuing_transaction_id
        self.updated = check_datetime(updated)
        self.created = check_datetime(created)


_resource = {"class": IssuingInvoice, "name": "IssuingInvoice"}


def create(invoice, user=None):
    """# Create an IssuingInvoice
    Send an IssuingInvoice object for creation at the Stark Infra API
    ## Parameters (required):
    - invoice [IssuingInvoice object]: IssuingInvoice object to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingInvoice object with updated attributes
    """
    return rest.post_single(resource=_resource, entity=invoice, user=user)


def get(id, user=None):
    """# Retrieve a specific IssuingInvoice
    Receive a single IssuingInvoice object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingInvoice object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, status=None, tags=None, user=None):
    """# Retrieve IssuingInvoices
    Receive a generator of IssuingInvoice objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "expired", "overdue", "paid"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of IssuingInvoice objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        status=status,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        limit=limit,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, tags=None, user=None):
    """# Retrieve IssuingInvoices
    Receive a list of IssuingInvoice objects previously created in the Stark Infra API and the cursor to the next page.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "expired", "overdue", "paid"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of IssuingInvoice objects with updated attributes
    - cursor to retrieve the next page of IssuingInvoice objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        status=status,
        after=check_date(after),
        before=check_date(before),
        tags=tags,
        limit=limit,
        user=user,
    )
    


from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_timedelta, check_datetime_or_date
from .__discount import Discount
from .__discount import resource as _discount_resource
from .__description import Description
from .__description import resource as _description_resource


class Invoice(Resource):
    """# creditnote.Invoice object
    Invoice issued after the contract is signed, to be paid by the credit receiver.
    ## Parameters (required):
    - amount [integer]: Invoice value in cents. Minimum = 1 (any value will be accepted). ex: 1234 (= R$ 12.34)
    ## Parameters (optional):
    - due [datetime.datetime or datetime.date or string, default now + 2 days]: Invoice due date in UTC ISO format. ex: "2020-10-28T17:59:26.249976+00:00" for immediate invoices and "2020-10-28" for scheduled invoices
    - expiration [integer or datetime.timedelta, default 5097600 (59 days)]: time interval in seconds between due date and expiration date. ex 123456789
    - tags [list of strings, default []]: list of strings for tagging
    - descriptions [list of creditnote.invoice.Description objects or dictionaries, default []]: list Description objects
    ## Attributes (return-only):
    - id [string]: unique id returned when Invoice is created. ex: "5656565656565656"
    - name [string]: payer name. ex: "Iron Bank S.A."
    - tax_id [string]: payer tax ID (CPF or CNPJ) with or without formatting. ex: "01234567890" or "20.018.183/0001-80"
    - pdf [string]: public Invoice PDF URL. ex: "https://invoice.starkbank.com/pdf/d454fa4e524441c1b0c1a729457ed9d8"
    - link [string]: public Invoice webpage URL. ex: "https://my-workspace.sandbox.starkbank.com/invoicelink/d454fa4e524441c1b0c1a729457ed9d8"
    - fine [float]: Invoice fine for overdue payment in %. ex: 2.5
    - interest [float]: Invoice monthly interest for overdue payment in %. ex: 5.2
    - nominal_amount [integer]: Invoice emission value in cents (will change if invoice is updated, but not if it's paid). ex: 400000
    - fine_amount [integer]: Invoice fine value calculated over nominal_amount. ex: 20000
    - interest_amount [integer]: Invoice interest value calculated over nominal_amount. ex: 10000
    - discount_amount [integer]: Invoice discount value calculated over nominal_amount. ex: 3000
    - discounts [list of creditnote.invoice.Discount objects]: list of Discount objects. ex: [Discount()]
    - brcode [string]: BR Code for the Invoice payment. ex: "00020101021226800014br.gov.bcb.pix2558invoice.starkbank.com/f5333103-3279-4db2-8389-5efe335ba93d5204000053039865802BR5913Arya Stark6009Sao Paulo6220051656565656565656566304A9A0"
    - status [string]: current Invoice status. ex: "registered" or "paid"
    - fee [integer]: fee charged by this Invoice. ex: 200 (= R$ 2.00)
    - transaction_ids [list of strings]: ledger transaction ids linked to this Invoice (if there are more than one, all but the first are reversals or failed reversal chargebacks). ex: ["19827356981273"]
    - created [datetime.datetime]: creation datetime for the Invoice. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the Invoice. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, due=None, expiration=None, tags=None, descriptions=None, id=None, name=None, tax_id=None,
                 pdf=None, link=None, fine=None, interest=None, nominal_amount=None, fine_amount=None,
                 interest_amount=None, discount_amount=None, discounts=None, brcode=None, status=None, fee=None,
                 transaction_ids=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.due = check_datetime_or_date(due)
        self.expiration = check_timedelta(expiration)
        self.fine = fine
        self.interest = interest
        self.tags = tags
        self.descriptions = parse_descriptions(descriptions)
        self.name = name
        self.tax_id = tax_id
        self.pdf = pdf
        self.link = link
        self.nominal_amount = nominal_amount
        self.fine_amount = fine_amount
        self.interest_amount = interest_amount
        self.discount_amount = discount_amount
        self.discounts = parse_discounts(discounts)
        self.brcode = brcode
        self.status = status
        self.fee = fee
        self.transaction_ids = transaction_ids
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


def parse_discounts(discounts):
    parsed_discounts = []
    if discounts is None:
        return discounts
    for discount in discounts:
        if isinstance(discount, Discount):
            parsed_discounts.append(discount)
            continue
        parsed_discounts.append(from_api_json(_discount_resource, discount))
    return parsed_discounts


def parse_descriptions(descriptions):
    parsed_descriptions = []
    if descriptions is None:
        return descriptions
    for description in descriptions:
        if isinstance(description, Description):
            parsed_descriptions.append(description)
            continue
        parsed_descriptions.append(from_api_json(_description_resource, description))
    return parsed_descriptions


_resource = {"class": Invoice, "name": "Invoice"}



from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class PixKey(Resource):
    """# PixKey object
    PixKeys link bank account information to key ids.
    Key ids are a convenient way to search and pass bank account information.
    When you initialize a Pix Key, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the created object.
    ## Parameters (required):
    - account_created [datetime.date, datetime.datetime or string]: opening Date or DateTime for the linked account. ex: "2022-01-01T12:00:00:00"
    - account_number [string]: number of the linked account. ex: "76543"
    - account_type [string]: type of the linked account. Options: "checking", "savings", "salary" or "payment"
    - branch_code [string]: branch code of the linked account. ex: 1234"
    - name [string]: holder's name of the linked account. ex: "Jamie Lannister"
    - tax_id [string]: holder's taxId (CPF/CNPJ) of the linked account. ex: "012.345.678-90"
    ## Parameters (optional):
    - id [string, default None]: id of the registered PixKey. Allowed types are: CPF, CNPJ, phone number or email. If this parameter is not passed, an EVP will be created. ex: "+5511989898989"
    - tags [list of strings, default []]]: list of strings for reference when searching for PixKeys. ex: ["employees", "monthly"]
    ## Attributes (return-only):
    - owned [datetime.datetime]: datetime when the key was owned by the holder. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - owner_type [string]: type of the owner of the PixKey. Options: "business" or "individual"
    - status [string]: current PixKey status. Options: "created", "registered", "canceled", "failed"
    - bank_code [string]: bank_code of the account linked to the Pix Key. ex: "20018183"
    - bank_name [string]: name of the bank that holds the account linked to the PixKey. ex: "StarkBank"
    - type [string]: type of the PixKey. Options: "cpf", "cnpj", "phone", "email" and "evp"
    - created [datetime.datetime]: creation datetime for the PixKey. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - statistics [list of Statistic objects]: list of objects with data regarding the Pix key statistics.
    - owner_statistics [list of Statistic objects]: list of objects with data regarding the Pix user statistics.
    """

    def __init__(self, account_created, account_number, account_type, branch_code, name, tax_id, id=None, tags=None,
                 owned=None, owner_type=None, status=None, bank_code=None, bank_name=None, type=None, created=None, statistics=None, owner_statistics=None):
        
        Resource.__init__(self, id=id)

        self.account_created = check_datetime(account_created)
        self.account_number = account_number
        self.account_type = account_type
        self.branch_code = branch_code
        self.name = name
        self.tax_id = tax_id
        self.tags = tags
        self.owned = check_datetime(owned)
        self.owner_type = owner_type
        self.status = status
        self.bank_code = bank_code
        self.bank_name = bank_name
        self.type = type
        self.created = check_datetime(created)
        self.statistics = statistics
        self.owner_statistics = owner_statistics


_resource = {"class": PixKey, "name": "PixKey"}


def create(key, user=None):
    """# Create a PixKey object
    Create a PixKey linked to a specific account in the Stark Infra API
    ## Parameters (optional):
    - key [PixKey object]: PixKey object to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixKey object with updated attributes.
    """
    return rest.post_single(resource=_resource, entity=key, user=user)


def get(id, payer_id, end_to_end_id=None, user=None, expand=None):
    """# Retrieve a PixKey object
    Retrieve the PixKey object linked to your Workspace in the Stark Infra API by its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    - payer_id [string]: tax id (CPF/CNPJ) of the individual or business requesting the PixKey information. This id is used by the Central Bank to limit request rates. ex: "20.018.183/0001-80"
    ## Parameters (optional):
    - end_to_end_id [string, default None]: central bank's unique transaction id. If the request results in the creation of a PixRequest, the same endToEndId should be used. If this parameter is not passed, one endToEndId will be automatically created. Example: "E00002649202201172211u34srod19le"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    - expand [list of strings, default None]: fields to expand information. ex: ["statistics", "owner_statistics"]
    ## Return:
    - PixKey object that corresponds to the given id.
    """

    return rest.get_id(id=id, payer_id=payer_id, end_to_end_id=end_to_end_id, resource=_resource, user=user, expand=expand)


def query(limit=None, after=None, before=None, status=None, tags=None, ids=None, type=None, user=None):
    """# Retrieve PixKeys
    Receive a generator of PixKey objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "registered", "canceled", "failed"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - type [string, default None]: filter for the type of retrieved PixKeys. Options: "cpf", "cnpj", "phone", "email" and "evp"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of PixKey objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        type=type,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, tags=None, ids=None, type=None,
         user=None):
    """# Retrieve paged PixKeys
    Receive a generator of PixKey objects previously created in the Stark Infra API
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call.
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "registered", "canceled", "failed"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - type [string, default None]: filter for the type of retrieved PixKeys. Options: "cpf", "cnpj", "phone", "email" and "evp"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - cursor to retrieve the next page of PixKey objects
    - generator of PixKey objects with updated attributes
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        type=type,
        user=user,
    )


def update(id, reason, account_created=None, account_number=None, account_type=None, branch_code=None, name=None,
           user=None):
    """# Update PixKey entity
    Update a PixKey parameters by passing id.
    ## Parameters (required):
    - id [string]: PixKey id. Allowed types are: CPF, CNPJ, phone number or email. ex: "5656565656565656"
    - reason [string]: reason why the PixKey is being patched. Options: "branchTransfer", "reconciliation", "userRequested" or "entryInvalid"
    ## Parameters (optional):
    - account_created [datetime.date, datetime.datetime or string, default None]: opening Date or DateTime for the account to be linked. ex: "2022-01-01"
    - account_number [string, default None]: number of the account to be linked. ex: "76543"
    - account_type [string, default None]: type of the account to be linked. Options: "checking", "savings", "salary" or "payment"
    - branch_code [string, default None]: branch code of the account to be linked. ex: 1234"
    - name [string, default None]: holder's name of the account to be linked. ex: "Jamie Lannister"
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixKey with updated attributes
    """
    payload = {
        "reason": reason,
        "account_created": account_created,
        "account_number": account_number,
        "account_type": account_type,
        "branch_code": branch_code,
        "name": name,
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def cancel(id, reason=None, user=None):
    """# Cancel a PixKey entity
    Cancel a PixKey entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    - reason [string, default userRequested]: reason why the PixKey is being cancelled. Options: "userRequested", "accountClosure", "fraud", "entryInvalid"
    ## Return:
    - canceled pixKey object
    """
    return rest.delete_id(resource=_resource, id=id, reason=reason, user=user)



from datetime import date, datetime
from .invoice.__invoice import Invoice
from .invoice.__invoice import _resource as _invoice_resource
from ..creditsigner.__creditsigner import CreditSigner
from ..creditsigner.__creditsigner import _resource as _creditsigner_resource
from .__transfer import Transfer
from .__transfer import _resource as _transfer_resource
from ..utils import rest
from starkcore.utils.api import from_api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date, check_datetime_or_date


class CreditNote(Resource):
    """# CreditNote object
    CreditNotes are used to generate CCB contracts between you and your customers.
    When you initialize a CreditNote, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the list of created objects.
    ## Parameters (required):
    - template_id [string]: ID of the contract template on which the CreditNote will be based. ex: "0123456789101112"
    - name [string]: credit receiver's full name. ex: "Edward Stark"
    - tax_id [string]: credit receiver's tax ID (CPF or CNPJ). ex: "20.018.183/0001-80"
    - scheduled [datetime.date, datetime.datetime or string]: date of transfer execution. ex: datetime(2020, 3, 10)
    - invoices [list of Invoice objects]: list of Invoice objects to be created and sent to the credit receiver. ex: [Invoice(), Invoice()]
    - payment [creditnote.Transfer]: payment entity to be created and sent to the credit receiver. ex: creditnote.Transfer()
    - signers [list of CreditSigner objects]: signer's name, contact and delivery method for the signature request. ex: [CreditSigner(), CreditSigner()]
    - external_id [string]: a string that must be unique among all your CreditNotes, used to avoid resource duplication. ex: "my-internal-id-123456"
    - street_line_1 [string]: credit receiver main address. ex: "Av. Paulista, 200"
    - street_line_2 [string]: credit receiver address complement. ex: "Apto. 123"
    - district [string]: credit receiver address district / neighbourhood. ex: "Bela Vista"
    - city [string]: credit receiver address city. ex: "Rio de Janeiro"
    - state_code [string]: credit receiver address state. ex: "GO"
    - zip_code [string]: credit receiver address zip code. ex: "01311-200"
    ## Parameters (conditionally required):
    - payment_type [string]: payment type, inferred from the payment parameter if it is not a dictionary. ex: "transfer"
    - nominal_amount [integer]: CreditNote value in cents. The nominal_amount parameter is required when amount is not sent. ex: 1234 (= R$ 12.34)
    - amount [integer]: amount in cents transferred to the credit receiver, before deductions. The amount parameter is required when nominal_amount is not sent. ex: 1234 (= R$ 12.34)
    ## Parameters (optional):
    - rebate_amount [integer, default 0]: credit analysis fee deducted from lent amount. ex: 11234 (= R$ 112.34)
    - tags [list of strings, default []]: list of strings for reference when searching for CreditNotes. ex: ["employees", "monthly"]
    - expiration [integer or datetime.timedelta, default 604800 (7 days)]: time interval in seconds between scheduled date and expiration date. ex 123456789
    ## Attributes (return-only):
    - id [string]: unique id returned when the CreditNote is created. ex: "5656565656565656"
    - document_id [string]: ID of the signed document to execute this CreditNote. ex: "4545454545454545"
    - status [string]: current status of the CreditNote. ex: "canceled", "created", "expired", "failed", "processing", "signed", "success"
    - transaction_ids [list of strings]: ledger transaction ids linked to this CreditNote. ex: ["19827356981273"]
    - workspace_id [string]: ID of the Workspace that generated this CreditNote. ex: "4545454545454545"
    - tax_amount [integer]: tax amount included in the CreditNote. ex: 100
    - nominal_interest [float]: yearly nominal interest rate of the CreditNote, in percentage. ex: 11.5
    - interest [float]: yearly effective interest rate of the CreditNote, in percentage. ex: 12.5
    - created [datetime.datetime]: creation datetime for the CreditNote. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the CreditNote. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, template_id, name, tax_id, scheduled, invoices, payment, signers, external_id,
                 street_line_1, street_line_2, district, city, state_code, zip_code, payment_type=None,
                 nominal_amount=None, amount=None, rebate_amount=None, tags=None, expiration=None, id=None,
                 document_id=None, status=None, transaction_ids=None, workspace_id=None, tax_amount=None,
                 nominal_interest=None, interest=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.template_id = template_id
        self.name = name
        self.tax_id = tax_id
        self.scheduled = check_datetime_or_date(scheduled)
        self.invoices = _parse_invoices(invoices)
        self.signers = _parse_signers(signers)
        self.external_id = external_id
        self.street_line_1 = street_line_1
        self.street_line_2 = street_line_2
        self.district = district
        self.city = city
        self.state_code = state_code
        self.zip_code = zip_code
        self.nominal_amount = nominal_amount
        self.amount = amount
        self.rebate_amount = rebate_amount
        self.tags = tags
        self.expiration = expiration
        self.document_id = document_id
        self.status = status
        self.transaction_ids = transaction_ids
        self.workspace_id = workspace_id
        self.tax_amount = tax_amount
        self.nominal_interest = nominal_interest
        self.interest = interest
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)

        self.payment, self.payment_type = _parse_payment(payment=payment, payment_type=payment_type)


_resource = {"class": CreditNote, "name": "CreditNote"}


def _parse_signers(signers):
    parsed_signers = []
    for signer in signers:
        if isinstance(signer, CreditSigner):
            parsed_signers.append(signer)
            continue
        parsed_signers.append(from_api_json(_creditsigner_resource, signer))
    return parsed_signers


def _parse_invoices(invoices):
    parsed_invoices = []
    for invoice in invoices:
        if isinstance(invoice, Invoice):
            parsed_invoices.append(invoice)
            continue
        parsed_invoices.append(from_api_json(_invoice_resource, invoice))
    return parsed_invoices


def _parse_payment(payment, payment_type):
    if isinstance(payment, dict):
        try:
            return from_api_json(*({
                "transfer": _transfer_resource,
            }[payment_type], payment)), payment_type
        except KeyError:
            return payment, payment_type

    if payment_type:
        return payment, payment_type

    if isinstance(payment, Transfer):
        return payment, "transfer"

    raise Exception(
        "payment must be either "
        "a dictionary"
        ", a starkinfra.creditnote.Transfer"
        ", but not a {}".format(type(payment))
    )


def create(notes, user=None):
    """# Create CreditNotes
    Send a list of CreditNote objects for creation at the Stark Infra API
    ## Parameters (required):
    - notes [list of CreditNote objects]: list of CreditNote objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of CreditNote objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=notes, user=user)


def get(id, user=None):
    """# Retrieve a specific CreditNote
    Receive a single CreditNote object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - CreditNote object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve CreditNotes
    Receive a generator of CreditNote objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["canceled", "created", "expired", "failed", "processing", "signed", "success"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of CreditNote objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, status=None, tags=None, ids=None, after=None, before=None, user=None):
    """# Retrieve paged CreditNotes
    Receive a list of up to 100 CreditNote objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. It must be an integer between 1 and 100. ex: 50
    - after [datetime.date or string, default None] date filter for objects created only after specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None] date filter for objects created only before specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["canceled", "created", "expired", "failed", "processing", "signed", "success"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of CreditNote objects with updated attributes
    - cursor to retrieve the next page of CreditNote objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        tags=tags,
        ids=ids,
        user=user,
    )


def cancel(id, user=None):
    """# Cancel a CreditNote entity
    Cancel a CreditNote entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: CreditNote unique id. ex: "6306109539221504"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - canceled CreditNote object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)


def pdf(noteId, user=None):
    """# Retrieve CCB disbursement
    Receive a CCB disbursement pdf file.
    ## Parameters (required):
    - noteId [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - Transfer pdf file
    """
    return rest.get_content(resource=_resource, id=noteId, user=user, sub_resource_name="payment/pdf")



from starkinfra.utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class IssuingTokenDesign(Resource):
    """# IssuingTokenDesign object
    The IssuingTokenDesign object displays the information of the token designs created in your Workspace.
    This resource represents the existent designs for the cards which will be tokenized.
    ## Attributes (return-only):
    - id [string]: unique id returned when IssuingTokenDesign is created. ex: "5656565656565656"
    - name [string]: Design name. ex: "Stark Bank - White Metal"
    - created [datetime.datetime]: creation datetime for the IssuingTokenDesign. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the IssuingTokenDesign. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, id=None, name=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.name = name
        self.created = created
        self.updated = updated


_resource = {"class": IssuingTokenDesign, "name": "IssuingTokenDesign"}


def get(id, user=None):
    """# Retrieve a specific IssuingTokenDesign
    Receive a single IssuingTokenDesign object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - IssuingTokenDesign object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, ids=None, user=None):
    """# Retrieve IssuingTokenDesigns
    Receive a generator of IssuingTokenDesign objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Max = 100. ex:
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of IssuingTokenDesigns objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        ids=ids,
        user=user,
    )


def page(cursor=None, limit=None, ids=None, user=None):
    """# Retrieve paged IssuingTokenDesign
    Receive a list of IssuingTokenDesign objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of IssuingTokenDesign objects with updated attributes
    - cursor to retrieve the next page of IssuingTokenDesign objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        ids=ids,
        user=user,
    )


def pdf(id, user=None):
    """# Retrieve a specific IssuingTokenDesign pdf file
    Receive a single IssuingTokenDesign pdf file generated in the Stark Infra API by its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - IssuingTokenDesign pdf file
    """
    return rest.get_content(resource=_resource, id=id, user=user, sub_resource_name="pdf")



from starkcore.utils.api import from_api_json
from starkcore.utils.subresource import SubResource
from ..utils import rest
from .__creditnotepreview import _sub_resource as _credit_note_preview_sub_resource, CreditNotePreview


class CreditPreview(SubResource):
    """# CreditPreview object
    A CreditPreview is used to get information from a credit before taking it.
    This resource can be used to preview credit notes
    ## Parameters (required):
    - credit [CreditNotePreview]: Information preview of the informed credit.
    - type [string]: Credit type. ex: "credit-note"
    """

    def __init__(self, type=None, credit=None):
        self.credit, self.type = _parse_credit(credit=credit, type=type)


_sub_resource = {"class": CreditPreview, "name": "CreditPreview"}


def create(previews, user=None):
    """# Create CreditPreviews
    Send a list of CreditPreview objects for processing in the Stark Infra API
    ## Parameters (required):
    - previews [list of CreditPreview objects]: list of CreditPreview objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of CreditPreview objects with updated attributes
    """
    return rest.post_multi(resource=_sub_resource, entities=previews, user=user)


def _parse_credit(credit, type):
    if isinstance(credit, dict):
        try:
            return from_api_json(*({
                "credit-note": _credit_note_preview_sub_resource,
            }[type], credit)), type
        except KeyError:
            return credit, type

    if type:
        return credit, type

    if isinstance(credit, CreditNotePreview):
        return credit, "credit-note"

    raise Exception(
        "credit must be either "
        "a dictionary"
        ", a starkinfra.creditpreview.CreditNotePreview"
        ", but not a {}".format(type(credit))
    )



from json import dumps
from starkcore.utils.api import api_json
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date
from ..utils import rest
from ..utils.parse import parse_and_verify


class PixReversal(Resource):
    """# PixReversal object
    PixReversals are instant payments used to revert PixRequests. You can only
    revert inbound PixRequests.
    When you initialize a PixReversal, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the list of created objects.
    ## Parameters (required):
    - amount [integer]: amount in cents to be reversed from the PixRequest. ex: 1234 (= R$ 12.34)
    - external_id [string]: string that must be unique among all your PixReversals. Duplicated external IDs will cause failures. By default, this parameter will block any PixReversal that repeats amount and receiver information on the same date. ex: "my-internal-id-123456"
    - end_to_end_id [string]: central bank's unique transaction ID. ex: "E79457883202101262140HHX553UPqeq"
    - reason [string]: reason why the PixRequest is being reversed. Options are "bankError", "fraud", "chashierError", "customerRequest"
    ## Parameters (optional):
    - tags [list of strings, default []]: list of strings for reference when searching for PixReversals. ex: ["employees", "monthly"]
    ## Attributes (return-only):
    - id [string]: unique id returned when the PixReversal is created. ex: "5656565656565656"
    - return_id [string]: central bank's unique reversal transaction ID. ex: "D20018183202202030109X3OoBHG74wo"
    - fee [string]: fee charged by this PixReversal. ex: 200 (= R$ 2.00)
    - status [string]: current PixReversal status. ex: "created", "processing", "success", "failed"
    - flow [string]: direction of money flow. ex: "in" or "out"
    - created [datetime.datetime]: creation datetime for the PixReversal. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the PixReversal. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self, amount, external_id, end_to_end_id, reason, tags=None, id=None, return_id=None,
                 fee=None, status=None, flow=None, description=None, created=None, updated=None):
        Resource.__init__(self, id=id)
        
        self.amount = amount
        self.external_id = external_id
        self.end_to_end_id = end_to_end_id
        self.reason = reason
        self.tags = tags
        self.return_id = return_id
        self.fee = fee
        self.status = status
        self.flow = flow
        self.description = description
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": PixReversal, "name": "PixReversal"}


def create(reversals, user=None):
    """# Create PixReversals
    Send a list of PixReversal objects for creation at the Stark Infra API
    ## Parameters (required):
    - reversals [list of PixReversal objects]: list of PixReversal objects to be created in the API
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of PixReversal objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=reversals, user=user)


def get(id, user=None):
    """# Retrieve a specific PixReversal
    Receive a single PixReversal object previously created in the Stark Infra API by its id
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixReversal object with updated attributes
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, status=None, ids=None, return_ids=None,
          external_ids=None, tags=None, user=None):
    """# Retrieve PixReversals
    Receive a generator of PixReversal objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "processing", "success", "failed"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - return_ids [list of strings, default None]: central bank's unique reversal transaction IDs. ex: ["D20018183202202030109X3OoBHG74wo", "D20018183202202030109X3OoBHG72rd"].
    - external_ids [list of strings, default None]: url safe strings that must be unique among all your PixReversals. Duplicated external IDs will cause failures. By default, this parameter will block any PixReversal that repeats amount and receiver information on the same date. ex: ["my-internal-id-123456", "my-internal-id-654321"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of PixReversal objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        return_ids=return_ids,
        external_ids=external_ids,
        tags=tags,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, ids=None,
         return_ids=None, external_ids=None, tags=None, user=None):
    """# Retrieve paged PixReversals
    Receive a list of up to 100 PixReversal objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your reversals.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "processing", "success", "failed"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - return_ids [list of strings, default None]: central bank's unique reversal transaction ID. ex: ["D20018183202202030109X3OoBHG74wo", "D20018183202202030109X3OoBHG72rd"].
    - external_ids [list of strings, default None]: url safe string that must be unique among all your PixReversals. Duplicated external IDs will cause failures. By default, this parameter will block any PixReversal that repeats amount and receiver information on the same date. ex: ["my-internal-id-123456", "my-internal-id-654321"]
    - tags [list of strings, default None]: tags to filter retrieved objects. ex: ["tony", "stark"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - list of PixReversal objects with updated attributes
    - cursor to retrieve the next page of PixReversal objects
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        return_ids=return_ids,
        external_ids=external_ids,
        tags=tags,
        user=user,
    )


def parse(content, signature, user=None):
    """# Create a single verified PixReversal object from a content string
    Create a single PixReversal object from a content string received from a handler listening at the reversal url.
    If the provided digital signature does not check out with the StarkInfra public key, a
    starkinfra.error.InvalidSignatureError will be raised.
    ## Parameters (required):
    - content [string]: response content from request received at user endpoint (not parsed)
    - signature [string]: base-64 digital signature received at response header "Digital-Signature"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - Parsed PixReversal object
    """
    request = parse_and_verify(
        content=content,
        signature=signature,
        user=user,
        resource=_resource
    )

    request.fee = request.fee or 0
    request.tags = request.tags or []
    request.external_id = request.external_id or ""
    request.description = request.description or ""
    
    return request


def response(status, reason=None):
    """# Helps you respond to a PixReversal authorization
    ## Parameters (required):
    - status [string]: response to the authorization. ex: "approved" or "denied"
    ## Parameters (conditionally required):
    - reason [string, default None]: denial reason. Required if the status is "denied". Options: "invalidAccountNumber", "blockedAccount", "accountClosed", "invalidAccountType", "invalidTransactionType", "taxIdMismatch", "invalidTaxId", "orderRejected", "reversalTimeExpired", "settlementFailed"
    ## Return:
    - Dumped JSON string that must be returned to us
    """
    params = {
        "authorization": {
            "status": status,
            "reason": reason,
        }
    }
    return dumps(api_json(params))



from .relay import set_relay
from starkcore.utils import rest


get_page = set_relay(rest.get_page)
get_stream = set_relay(rest.get_stream)
get_id = set_relay(rest.get_id)
get_content = set_relay(rest.get_content)
get_sub_resource = set_relay(rest.get_sub_resource)
get_sub_resources = set_relay(rest.get_sub_resources)
post_multi = set_relay(rest.post_multi)
put_multi = set_relay(rest.put_multi)
post_single = set_relay(rest.post_single)
delete_id = set_relay(rest.delete_id)
patch_id = set_relay(rest.patch_id)
get_raw = set_relay(rest.get_raw)
post_raw = set_relay(rest.post_raw)
put_raw = set_relay(rest.put_raw)
patch_raw = set_relay(rest.patch_raw)
delete_raw = set_relay(rest.delete_raw)


from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_datetime, check_date


class IssuingBillingInvoice(Resource):
    """# IssuingBillingInvoice object
    Check out our API Documentation at https://starkinfra.com/docs/api#issuing-billing-invoice
    """

    def __init__(self, id=None, name=None, tax_id=None, fine=None, interest=None, status=None, amount=None,
                 nominal_amount=None, brcode=None, link=None, due=None, start=None, end=None, created=None,
                 updated=None):
        Resource.__init__(self, id=id)

        self.tax_id = tax_id
        self.name = name
        self.fine = fine
        self.interest = interest
        self.status = status
        self.amount = amount
        self.nominal_amount = nominal_amount
        self.brcode = brcode
        self.link = link
        self.due = check_datetime(due)
        self.start = check_datetime(start)
        self.end = check_datetime(end)
        self.created = check_datetime(created)
        self.updated = check_datetime(updated)


_resource = {"class": IssuingBillingInvoice, "name": "IssuingBillingInvoice"}


def get(id, user=None):
    """# IssuingBillingInvoice object
    Check out our API Documentation at https://starkinfra.com/docs/api#issuing-billing-invoice
    """
    return rest.get_id(resource=_resource, id=id, user=user)


def query(limit=None, after=None, before=None, status=None, id=None, tags=None, user=None):
    """# IssuingBillingInvoice object
    Check out our API Documentation at https://starkinfra.com/docs/api#issuing-billing-invoice
    """
    return rest.get_stream(
        resource=_resource,
        status=status,
        after=check_datetime(after),
        before=check_datetime(before),
        id=id,
        tags=tags,
        limit=limit,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, tags=None, user=None):
    """# IssuingBillingInvoice object
    Check out our API Documentation at https://starkinfra.com/docs/api#issuing-billing-invoice
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        status=status,
        after=check_datetime(after),
        before=check_datetime(before),
        tags=tags,
        limit=limit,
        user=user,
    )



from starkcore.utils.subresource import SubResource
from starkinfra.utils import rest


class MerchantCategory(SubResource):
    """# MerchantCategory object
    MerchantCategory's codes and types are used to define categories filters in IssuingRules.
    A MerchantCategory filter must define exactly one parameter between code and type.
    A type, such as "food", "services", etc., defines an entire group of merchant codes,
    whereas a code only specifies a specific MCC.
    ## Parameters (conditionally required):
    - code [string, default None]: category's code. ex: "veterinaryServices", "fastFoodRestaurants"
    - type [string, default None]: category's type. ex: "pets", "food"
    ## Attributes (return-only):
    - name [string]: category's name. ex: "Veterinary services", "Fast food restaurants"
    - number [string]: category's number. ex: "742", "5814"
    """

    def __init__(self, code=None, type=None, name=None, number=None):
        self.code = code
        self.type = type
        self.name = name
        self.number = number


_resource = {"class": MerchantCategory, "name": "MerchantCategory"}


def query(search=None, user=None):
    """# Retrieve MerchantCategories
    Receive a generator of MerchantCategory objects previously created in the Stark Infra API
    ## Parameters (optional):
    - search [string, default None]: keyword to search for code, type, name or number
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call
    ## Return:
    - generator of MerchantCategory objects with updated attributes
    """
    return rest.get_stream(
        resource=_resource,
        search=search,
        user=user,
    )
    


from ..utils import rest
from starkcore.utils.resource import Resource
from starkcore.utils.checks import check_date


class PixChargeback(Resource):
    """# PixChargeback object
    A Pix chargeback can be created when fraud is detected on a transaction or a system malfunction
    results in an erroneous transaction.
    It notifies another participant of your request to reverse the payment they have received.
    When you initialize a PixChargeback, the entity will not be automatically
    created in the Stark Infra API. The 'create' function sends the objects
    to the Stark Infra API and returns the created object.
    ## Parameters (required):
    - amount [integer]: amount in cents to be reversed. ex: 11234 (= R$ 112.34)
    - reference_id [string]: end_to_end_id or return_id of the transaction to be reversed. ex: "E20018183202201201450u34sDGd19lz"
    - reason [string]: reason why the reversal was requested. Options: "fraud", "flaw", "reversalChargeback"
    ## Parameters (conditionally required):
    - description [string, default None]: description for the PixChargeback. Required if reason is "flaw".
    ## Parameters (optional):
    - tags [list of strings, default None]: list of strings for tagging. ex: ["travel", "food"]
    ## Attributes (return-only):
    - id [string]: unique id returned when the PixChargeback is created. ex: "5656565656565656"
    - bacen_id [string, default None]: unique transaction id returned from Central Bank. ex: "ccf9bd9c-e99d-999e-bab9-b999ca999f99"
    - analysis [string]: analysis that led to the result.
    - sender_bank_code [string]: bank_code of the Pix participant that created the PixChargeback. ex: "20018183"
    - receiver_bank_code [string]: bank_code of the Pix participant that received the PixChargeback. ex: "20018183"
    - rejection_reason [string]: reason for the rejection of the Pix chargeback. Options: "noBalance", "accountClosed", "invalidRequest", "unableToReverse"
    - reversal_reference_id [string]: return_id or end_to_end_id of the reversal transaction. ex: "D20018183202202030109X3OoBHG74wo"
    - result [string]: result after the analysis of the PixChargeback by the receiving party. Options: "rejected", "accepted", "partiallyAccepted"
    - flow [string]: direction of the Pix Chargeback. Options: "in" for received chargebacks, "out" for chargebacks you requested
    - status [string]: current PixChargeback status. Options: "created", "failed", "delivered", "closed", "canceled"
    - created [datetime.datetime]: creation datetime for the PixChargeback. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    - updated [datetime.datetime]: latest update datetime for the PixChargeback. ex: datetime.datetime(2020, 3, 10, 10, 30, 0, 0)
    """

    def __init__(self,  amount, reference_id, reason, description=None, tags=None, id=None, bacen_id=None, analysis=None,
                 sender_bank_code=None, receiver_bank_code=None, rejection_reason=None, reversal_reference_id=None,
                 result=None, flow=None, status=None, created=None, updated=None):
        Resource.__init__(self, id=id)

        self.amount = amount
        self.reference_id = reference_id
        self.reason = reason
        self.description = description
        self.tags = tags
        self.bacen_id = bacen_id
        self.analysis = analysis
        self.sender_bank_code = sender_bank_code
        self.receiver_bank_code = receiver_bank_code
        self.rejection_reason = rejection_reason
        self.reversal_reference_id = reversal_reference_id
        self.result = result
        self.flow = flow
        self.status = status
        self.created = created
        self.updated = updated


_resource = {"class": PixChargeback, "name": "PixChargeback"}


def create(chargebacks, user=None):
    """# Create PixChargeback objects
    Create PixChargebacks in the Stark Infra API
    ## Parameters (optional):
    - chargebacks [list of PixChargeback]: list of PixChargeback objects to be created in the API.
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - list of PixChargeback objects with updated attributes
    """
    return rest.post_multi(resource=_resource, entities=chargebacks, user=user)


def get(id, user=None):
    """# Retrieve a PixChargeback object
    Retrieve the PixChargeback object linked to your Workspace in the Stark Infra API using its id.
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixChargeback object that corresponds to the given id.
    """

    return rest.get_id(id=id, resource=_resource, user=user)


def query(limit=None, after=None, before=None, status=None, ids=None, bacen_id=None, flow=None, tags=None, user=None):
    """# Retrieve PixChargebacks
    Receive a generator of PixChargeback objects previously created in the Stark Infra API
    ## Parameters (optional):
    - limit [integer, default None]: maximum number of objects to be retrieved. Unlimited if None. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "failed", "delivered", "closed", "canceled"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - bacen_id [string, default None]: unique transaction id returned from Central Bank. ex: "ccf9bd9c-e99d-999e-bab9-b999ca999f99"
    - flow [string, default None]: direction of the Pix Chargeback. Options: "in" for received chargebacks, "out" for chargebacks you requested
    - tags [list of strings, default None]: filter for tags of retrieved objects. ex: ["travel", "food"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - generator of PixChargeback objects with updated attributes
    """

    return rest.get_stream(
        resource=_resource,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        bacen_id=bacen_id,
        flow=flow,
        tags=tags,
        user=user,
    )


def page(cursor=None, limit=None, after=None, before=None, status=None, ids=None, bacen_id=None, flow=None, tags=None, user=None):
    """# Retrieve PixChargebacks
    Receive a list of up to 100 PixChargeback objects previously created in the Stark Infra API and the cursor to the next page.
    Use this function instead of query if you want to manually page your requests.
    ## Parameters (optional):
    - cursor [string, default None]: cursor returned on the previous page function call.
    - limit [integer, default 100]: maximum number of objects to be retrieved. Max = 100. ex: 35
    - after [datetime.date or string, default None]: date filter for objects created after a specified date. ex: datetime.date(2020, 3, 10)
    - before [datetime.date or string, default None]: date filter for objects created before a specified date. ex: datetime.date(2020, 3, 10)
    - status [list of strings, default None]: filter for status of retrieved objects. ex: ["created", "failed", "delivered", "closed", "canceled"]
    - ids [list of strings, default None]: list of ids to filter retrieved objects. ex: ["5656565656565656", "4545454545454545"]
    - bacen_id [string, default None]: unique transaction id returned from Central Bank. ex: "ccf9bd9c-e99d-999e-bab9-b999ca999f99"
    - flow [string, default None]: direction of the Pix Chargeback. Options: "in" for received chargebacks, "out" for chargebacks you requested
    - tags [list of strings, default None]: filter for tags of retrieved objects. ex: ["travel", "food"]
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - cursor to retrieve the next page of PixChargeback objects
    - generator of PixChargeback objects with updated attributes
    """
    return rest.get_page(
        resource=_resource,
        cursor=cursor,
        limit=limit,
        after=check_date(after),
        before=check_date(before),
        status=status,
        ids=ids,
        bacen_id=bacen_id,
        flow=flow,
        tags=tags,
        user=user,
    )


def update(id, result, rejection_reason=None, reversal_reference_id=None, analysis=None, user=None):
    """# Update PixChargeback entity
    Respond to a received PixChargeback.
    ## Parameters (required):
    - id [string]: PixChargeback id. ex: '5656565656565656'
    - result [string]: result after the analysis of the PixChargeback. Options: "rejected", "accepted", "partiallyAccepted"
    ## Parameters (conditionally required):
    - rejection_reason [string, default None]: if the PixChargeback is rejected a reason is required. Options: "noBalance", "accountClosed", "invalidRequest", "unableToReverse"
    - analysis [string, default None]: description of the analysis that led to the result. Required if rejection_reason is "invalidRequest".
    - reversal_reference_id [string, default None]: return_id of the reversal transaction. ex: "D20018183202201201450u34sDGd19lz"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - PixChargeback with updated attributes
    """
    payload = {
        "result": result,
        "rejection_reason": rejection_reason,
        "reversal_reference_id": reversal_reference_id,
        "analysis": analysis
    }
    return rest.patch_id(resource=_resource, id=id, user=user, payload=payload)


def cancel(id, user=None):
    """# Cancel a PixChargeback entity
    Cancel a PixChargeback entity previously created in the Stark Infra API
    ## Parameters (required):
    - id [string]: object unique id. ex: "5656565656565656"
    ## Parameters (optional):
    - user [Organization/Project object, default None]: Organization or Project object. Not necessary if starkinfra.user was set before function call.
    ## Return:
    - canceled PixChargeback object
    """
    return rest.delete_id(resource=_resource, id=id, user=user)



from .bacenid import create as _create_bacen_id


def create(bank_code):
    """
    Generates a random return-id based on your bank code (ISPB)
    ## Parameters (required):
    - bank_code [string]: Your bank code (ISPB). ex: "20018183"
    ## Return:
    - Random returnId based on your bank code.
    """
    return "D" + _create_bacen_id(bank_code)
